/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 10/07/23.
//

#ifndef FGGL_COMPONENTS_CORE_SERVICES_REGISTRY_HPP
#define FGGL_COMPONENTS_CORE_SERVICES_REGISTRY_HPP

#include "service.hpp"

#include <memory>
#include <map>

namespace fggl::services {


	template<typename T, typename U>
	concept Subclass = std::is_base_of<U, T>::value;

	class Registry {
		public:
			void provideUnsafe(ServiceName api, std::shared_ptr<void> service) {
				m_services[ api ] = service;
			}

			inline bool contains(ServiceName service) const {
				return m_services.find(service) != m_services.end();
			}

			template<ServiceAPI svc>
			Service<svc> get() const {
				auto wrapper = m_services.at( svc::SERVICE_ID );
				auto ptr = std::static_pointer_cast<svc>(wrapper);
				return Service<svc>(ptr);
			}

		private:
			std::map<ServiceName, std::shared_ptr<void>> m_services;
	};

} // namespace fggl::services

#endif //FGGL_COMPONENTS_CORE_SERVICES_REGISTRY_HPP
