/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 24/06/23.
//

#ifndef FGGL_COMPONENTS_PHYSICS_INCLUDE_PHYS_MODULE_HPP
#define FGGL_COMPONENTS_PHYSICS_INCLUDE_PHYS_MODULE_HPP

	#include "fggl/services/module.hpp"
	#include <fggl/phys/types.hpp>
	#include <fggl/phys/service.hpp>


namespace fggl::phys {

	class InternalPhysicsEngine : public PhysicsEngine {
		public:
			explicit InternalPhysicsEngine( entity::EntityManager* entities );
			void step() override;

			std::vector<ContactPoint> scanCollisions(entity::EntityID) override;

			entity::EntityID raycast(math::vec3, math::vec3) override;
			std::vector<entity::EntityID> raycastAll(math::vec3, math::vec3) override;
			std::vector<entity::EntityID> sweep(PhyShape &shape,
														math::Transform &from,
														math::Transform &to) override;

			inline void setDebugDraw(bool enable) override {
				m_debugDraw = enable;
			}

		private:
			entity::EntityManager* m_entities;
			bool m_debugDraw = false;
	};

	class InternalPhysicsProvider : public PhysicsProvider {
		public:
			virtual ~InternalPhysicsProvider() = default;
			InternalPhysicsEngine* create(entity::EntityManager*, entity::EntityFactory*) override;
	};

	struct InternalPhysics : public services::Module {
		FGGL_MODULE(fggl::phys::Internal, fggl::entity::ECS)

		void wireServices(services::ModuleBinder& binder) override;
	};

} // namespace fggl::phys

#endif //FGGL_COMPONENTS_PHYSICS_INCLUDE_PHYS_MODULE_HPP
