/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#include "fggl/audio/openal/audio.hpp"
#include "fggl/assets/types.hpp"
#include "fggl/assets/manager.hpp"

#include "../../stb/stb_vorbis.h"

namespace fggl::audio::openal {

	auto load_vorbis(assets::Loader* /*loader*/, const assets::AssetID &guid, const assets::LoaderContext &data, void* userPtr) -> assets::AssetRefRaw {
		auto *manager = static_cast<assets::AssetManager*>(userPtr);
		auto filePath = data.assetPath;

		// vorbis
		auto* clip = new AudioClipShort();
		clip->sampleCount = stb_vorbis_decode_filename( filePath.c_str(), &clip->channels, &clip->sampleRate, &clip->data);
		debug::info("clip loaded: channels={}, sampleRate={}, sampleCount={}", clip->channels, clip->sampleRate, clip->sampleCount);

		if ( clip->sampleCount == -1 ) {
			return nullptr;
		}

		manager->set(guid, clip);
		return nullptr;
	}

	auto load_vorbis_short(std::filesystem::path path, assets::MemoryBlock& /*block*/) -> bool {
		// vorbis
		auto* clip = new AudioClipShort();
		clip->sampleCount = stb_vorbis_decode_filename( path.c_str(), &clip->channels, &clip->sampleRate, &clip->data);
		debug::info("clip loaded: channels={}, sampleRate={}, sampleCount={}", clip->channels, clip->sampleRate, clip->sampleCount);

		return clip->sampleCount != 1;
	}

	auto check_vorbis(const std::filesystem::path& path ) -> assets::AssetTypeID {
		if ( path.extension() != ".ogg" ) {
			return assets::INVALID_ASSET_TYPE;
		}

		auto* clip = new AudioClipShort();
		clip->sampleCount = stb_vorbis_decode_filename( path.c_str(), &clip->channels, &clip->sampleRate, &clip->data);

		if ( clip->sampleCount == -1 ) {
			return assets::INVALID_ASSET_TYPE;
		}

		delete clip;
		return ASSET_CLIP_SHORT;
	}

	void AudioSource::play(const AudioClipShort &clip, bool looping) {
		check_error("pre play");

		AudioType format = clip.channels == 1 ? AudioType::MONO_16 : AudioType::STEREO_16;
		m_splat.setData(format, clip.data, clip.size(), clip.sampleRate);
		check_error("saving to buffer");

		play(m_splat, looping);
		check_error("post play");
	}

	void AudioServiceOAL::play(const std::string &filename, bool looping) {
		debug::info("beginning audio: {}", filename);

		// load audio clip into temp storage
		auto assetRef = assets::make_asset_id_rt("core", filename);
		auto* clip = m_assets->get<AudioClipShort>(assetRef);
		if ( clip == nullptr ) {
			debug::warning("audio asset requested, but not loaded: {}", filename);
			return;
		}

		play(*clip, looping);
		debug::info("played audio: {}", filename);
	}

	void AudioServiceOAL::play(const AudioClipShort &clip, bool looping) {
		if ( m_defaultSource == nullptr ){
			return;
		}

		// play the audio on the default (non-positioned) source
		m_defaultSource->play(clip, looping);
	}

	void AudioServiceOAL::release() {
		m_defaultSource = nullptr;

		alcMakeContextCurrent(nullptr);
		alcDestroyContext(m_context);
		alcCloseDevice(m_device);
		m_context = nullptr;
		m_device = nullptr;
	}

}