/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 11/03/23.
//

#include "fggl/gui/model/parser.hpp"
#include <yaml-cpp/yaml.h>

namespace fggl::gui::model {

	Widget* YamlToWidgetTree(WidgetFactory& factory, const YAML::Node& config) {
		Widget* root;
		if ( config["template"] ) {
			root = factory.build( config["template"].as<std::string>());
		} else {
			root = factory.buildEmpty();
		}

		// deal with attrs
		for ( auto attr : config["attrs"] ) {
			root->set(attr.first.as<std::string>(), attr.second.as<std::string>());
		}

		// deal with child nodes
		for ( auto child : config["children"] ) {
			Widget* childWidget = YamlToWidgetTree(factory, child);
			root->addChild(*childWidget);
		}

		// are we a template definition?
		if ( config["define"] ) {
			factory.push( config["define"].as<std::string>(), std::move(*root) );
			return factory.getTemplate( config["define"].as<std::string>() );
		}

		return root;
	}

	inline Widget* parseFile(WidgetFactory& factory, const std::string& path) {
		YAML::Node root = YAML::LoadFile(path);
		if ( !root ){
			return nullptr;
		}
		return YamlToWidgetTree(factory, root);
	}

}