/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 11/03/23.
//

#include "fggl/gui/model/structure.hpp"
#include "fggl/gui/fonts.hpp"

#include <algorithm>
#include <utility>
#include <string>

namespace fggl::gui::model {

	math::vec2 calcTextBounds(const std::string& value, std::shared_ptr<FontFace> face) {
		if ( face == nullptr ){
			debug::warning("No preferred font sent, cowardly refusing to process text");
			return {};
		}

		math::vec2 max{0, 0};
		for (auto letter : value) {
			auto metrics = face->metrics(letter);

			max.x += metrics.size.x + (metrics.advance >> 6);
			max.y = std::max(max.y, metrics.size.y);
		}
		return max;
	}

	inline math::vec2 calcBoxContrib(const Widget& widget, const std::string& name) {
		return math::vec2{
			widget.get_or_default<float>( name + "::left") + widget.get_or_default<float>(name + "::right"),
			widget.get_or_default<float>( name + "::top" ) + widget.get_or_default<float>( name + "::bottom" )
		};
	}

	void Widget::calcPrefSize(std::shared_ptr<FontFace> face) {
		if ( !m_dirty ){
			return;
		}

		auto padding = calcBoxContrib( *this, "padding");
		auto border = calcBoxContrib( *this, "border");
		auto content = math::vec2{0,0};

		if (hasAttr("text")) {
			content += calcTextBounds( get<std::string>("text"), std::move(face) );
		}

		m_cachedSize = padding + content + content;
		debug::info("my preferred size is: ({}, {})", m_cachedSize.x, m_cachedSize.y);

		m_dirty = false;
	}

}