/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 19/12/22.
//

#ifndef FGGL_GRID_ACTIONS_H
#define FGGL_GRID_ACTIONS_H

#include <cstdint>
#include <variant>

#include "fggl/grid/hexagon.hpp"
#include "fggl/grid/tokens.hpp"

#include "fggl/util/safety.hpp"

namespace fggl::grid {

	template<typename State, typename Target>
	class ActionType;

	template<typename State, typename Target>
	class Action {
		public:
			virtual void progress(State* state) = 0;
			virtual bool isDone() = 0;
	};

	template<typename State, typename Target>
	class InstantAction : public Action<State, Target> {
		public:
			void progress(State* state) final;
			bool isDone() final;
		private:
			ActionType<State, Target>* m_action;
			TokenIdentifer m_actor;
			Target m_target;
	};

	template<typename State, typename Target>
	class DurativeAction : public Action<State, Target> {
		public:
			void progress(State* state) final;
			bool isDone() final;
		private:
			ActionType<State, Target>* m_action;
			TokenIdentifer m_actor;
			Target m_target;
			uint64_t m_completionRound;
	};

	template<typename State, typename Target>
	class ActionType {
		public:
			virtual bool canApply(const State* state, TokenIdentifer actor, Target target) = 0;
			virtual void apply(State* state, TokenIdentifer actor, Target target) = 0;

			virtual Action<State, Target> ground(TokenIdentifer actor, Target target) = 0;
	};


} // namespace fggl::grid

#endif //FGGL_GRID_ACTIONS_H
